@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtPositioning module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -63,21 +55,40 @@ QT_BEGIN_NAMESPACE
 
     The circle is considered invalid if the center coordinate is invalid
     or if the radius is less than zero.
-*/
 
-#ifdef Q_NO_USING_KEYWORD
-/*!
-    \fn bool QGeoCircle::operator==(const QGeoShape &other) const
-
-    Returns true if \a other is equal to this geo circle; otherwise returns false.
+    This class is a \l Q_GADGET since Qt 5.5.  It can be
+    \l{Cpp_value_integration_positioning}{directly used from C++ and QML}.
 */
 
 /*!
-    \fn bool QGeoCircle::operator!=(const QGeoShape &other) const
+    \property QGeoCircle::center
+    \brief This property holds the center coordinate for the geo circle.
 
-    Returns true if \a other is not equal to this geo circle; otherwise returns false.
+    The circle is considered invalid if this property contains an invalid
+    coordinate.
+
+    A default constructed QGeoCircle uses an invalid \l QGeoCoordinate
+    as center.
+
+    While this property is introduced in Qt 5.5, the related accessor functions
+    exist since the first version of this class.
+
+    \since 5.5
+*/
+
+/*!
+    \property QGeoCircle::radius
+    \brief This property holds the circle radius in meters.
+
+    The circle is considered invalid if this property is negative.
+
+    By default, the radius is initialized with \c -1.
+
+    While this property is introduced in Qt 5.5, the related accessor functions
+    exist since the first version of this class.
+
+    \since 5.5
 */
-#endif
 
 inline QGeoCirclePrivate *QGeoCircle::d_func()
 {
@@ -89,12 +100,24 @@ inline const QGeoCirclePrivate *QGeoCircle::d_func() const
     return static_cast<const QGeoCirclePrivate *>(d_ptr.constData());
 }
 
+struct CircleVariantConversions
+{
+    CircleVariantConversions()
+    {
+        QMetaType::registerConverter<QGeoShape, QGeoCircle>();
+        QMetaType::registerConverter<QGeoCircle, QGeoShape>();
+    }
+};
+
+Q_GLOBAL_STATIC(CircleVariantConversions, initCircleConversions)
+
 /*!
     Constructs a new, invalid geo circle.
 */
 QGeoCircle::QGeoCircle()
 :   QGeoShape(new QGeoCirclePrivate)
 {
+    initCircleConversions();
 }
 
 /*!
@@ -102,6 +125,7 @@ QGeoCircle::QGeoCircle()
 */
 QGeoCircle::QGeoCircle(const QGeoCoordinate &center, qreal radius)
 {
+    initCircleConversions();
     d_ptr = new QGeoCirclePrivate(center, radius);
 }
 
@@ -111,6 +135,7 @@ QGeoCircle::QGeoCircle(const QGeoCoordinate &center, qreal radius)
 QGeoCircle::QGeoCircle(const QGeoCircle &other)
 :   QGeoShape(other)
 {
+    initCircleConversions();
 }
 
 /*!
@@ -119,6 +144,7 @@ QGeoCircle::QGeoCircle(const QGeoCircle &other)
 QGeoCircle::QGeoCircle(const QGeoShape &other)
 :   QGeoShape(other)
 {
+    initCircleConversions();
     if (type() != QGeoShape::CircleType)
         d_ptr = new QGeoCirclePrivate;
 }
@@ -159,7 +185,7 @@ bool QGeoCircle::operator!=(const QGeoCircle &other) const
 
 bool QGeoCirclePrivate::isValid() const
 {
-    return center.isValid() && !qIsNaN(radius) && radius >= -1e-7;
+    return m_center.isValid() && !qIsNaN(radius) && radius >= -1e-7;
 }
 
 bool QGeoCirclePrivate::isEmpty() const
@@ -174,17 +200,17 @@ void QGeoCircle::setCenter(const QGeoCoordinate &center)
 {
     Q_D(QGeoCircle);
 
-    d->center = center;
+    d->m_center = center;
 }
 
 /*!
-    Returns the center coordinate of this geo circle.
+    Returns the center coordinate of this geo circle. Equivalent to QGeoShape::center().
 */
 QGeoCoordinate QGeoCircle::center() const
 {
     Q_D(const QGeoCircle);
 
-    return d->center;
+    return d->center();
 }
 
 /*!
@@ -212,7 +238,17 @@ bool QGeoCirclePrivate::contains(const QGeoCoordinate &coordinate) const
     if (!isValid() || !coordinate.isValid())
         return false;
 
-    return center.distanceTo(coordinate) <= radius;
+    // see QTBUG-41447 for details
+    qreal distance = m_center.distanceTo(coordinate);
+    if (qFuzzyCompare(distance, radius) || distance <= radius)
+        return true;
+
+    return false;
+}
+
+QGeoCoordinate QGeoCirclePrivate::center() const
+{
+    return m_center;
 }
 
 /*!
@@ -223,7 +259,7 @@ void QGeoCirclePrivate::extendShape(const QGeoCoordinate &coordinate)
     if (!isValid() || !coordinate.isValid() || contains(coordinate))
         return;
 
-    radius = center.distanceTo(coordinate);
+    radius = m_center.distanceTo(coordinate);
 }
 
 /*!
@@ -238,8 +274,8 @@ void QGeoCircle::translate(double degreesLatitude, double degreesLongitude)
 
     Q_D(QGeoCircle);
 
-    double lat = d->center.latitude();
-    double lon = d->center.longitude();
+    double lat = d->m_center.latitude();
+    double lon = d->m_center.longitude();
 
     lat += degreesLatitude;
     lon += degreesLongitude;
@@ -265,7 +301,7 @@ void QGeoCircle::translate(double degreesLatitude, double degreesLongitude)
             lon -= 180;
     }
 
-    d->center = QGeoCoordinate(lat, lon);
+    d->m_center = QGeoCoordinate(lat, lon);
 }
 
 /*!
@@ -284,6 +320,25 @@ QGeoCircle QGeoCircle::translated(double degreesLatitude, double degreesLongitud
     return result;
 }
 
+/*!
+    Returns the geo circle properties as a string.
+
+    \since 5.5
+*/
+
+QString QGeoCircle::toString() const
+{
+    if (type() != QGeoShape::CircleType) {
+        qWarning("Not a circle");
+        return QStringLiteral("QGeoCircle(not a circle)");
+    }
+
+    return QStringLiteral("QGeoCircle({%1, %2}, %3)")
+        .arg(center().latitude())
+        .arg(center().longitude())
+        .arg(radius());
+}
+
 /*******************************************************************************
 *******************************************************************************/
 
@@ -293,12 +348,12 @@ QGeoCirclePrivate::QGeoCirclePrivate()
 }
 
 QGeoCirclePrivate::QGeoCirclePrivate(const QGeoCoordinate &center, qreal radius)
-:   QGeoShapePrivate(QGeoShape::CircleType), center(center), radius(radius)
+:   QGeoShapePrivate(QGeoShape::CircleType), m_center(center), radius(radius)
 {
 }
 
 QGeoCirclePrivate::QGeoCirclePrivate(const QGeoCirclePrivate &other)
-:   QGeoShapePrivate(QGeoShape::CircleType), center(other.center),
+:   QGeoShapePrivate(QGeoShape::CircleType), m_center(other.m_center),
     radius(other.radius)
 {
 }
@@ -317,7 +372,7 @@ bool QGeoCirclePrivate::operator==(const QGeoShapePrivate &other) const
 
     const QGeoCirclePrivate &otherCircle = static_cast<const QGeoCirclePrivate &>(other);
 
-    return radius == otherCircle.radius && center == otherCircle.center;
+    return radius == otherCircle.radius && m_center == otherCircle.m_center;
 }
 
 QT_END_NAMESPACE
